import axios from 'axios'
import {Message} from 'element-ui'
import store from '@/store'
import {getToken} from '@/utils/auth'
// create an axios instance
const service = axios.create({
  baseURL: process.env.VUE_APP_BASE_API, // url = base url + request url
  withCredentials: true, // send cookies when cross-domain requests
  timeout: 15000, // request timeout,
  headers: {
    'content-type': 'application/json'
  }
})


const requests = {}

// request interceptor
service.interceptors.request.use(
  config => {
    // do something before request is sent
    if (store.getters.token) {
      // let each request carry token
      // ['X-Token'] is a custom headers key
      // please modify it according to the actual situation

      config.headers['access_token'] = getToken()

      config.headers['Authorization'] = 'Bearer ' + getToken()
    }

    const isRepeatSubmit = (config.headers || {}).repeatSubmit === false
    if (!isRepeatSubmit && (config.method === 'post' || config.method === 'put')) {
      const requestObj = {
        name: config.url + '_' + config.method,
        url: config.url,
        data: typeof config.data === 'object' ? JSON.stringify(config.data) : config.data,
        time: new Date().getTime()
      }

      if (!requests[requestObj.name]) {
        requests[requestObj.name] = requestObj
      } else {
        const sessionObj = requests[requestObj.name]
        const s_url = sessionObj.url                // 请求地址
        const s_data = sessionObj.data;                // 请求数据
        const s_time = sessionObj.time;                // 请求时间
        const interval = 1000;                         // 间隔时间(ms)，小于此时间视为重复提交
        if (s_data === requestObj.data && requestObj.time - s_time < interval && s_url === requestObj.url) {
          const message = '数据正在处理，请勿重复提交';
          console.warn(`[${s_url}]: ` + message)
          Message({
            message: '数据正在处理，请勿重复提交',
            type: 'error',
            duration: 5 * 1000
          })
          return Promise.reject(new Error(message))
        } else {
          requests[requestObj.name] = requestObj
        }
      }
    }

    return config
  },
  error => {
    // do something with request error
    console.log(error) // for debug
    return Promise.reject(error)
  }
)

// response interceptor
service.interceptors.response.use(
  /**
   * If you want to get http information such as headers or status
   * Please return  response => response
   */

  /**
   * Determine the request status by custom code
   * Here is just an example
   * You can also judge the status by HTTP Status Code
   */
  response => {
    const res = response.data
    // if the custom code is not 20000, it is judged as an error.
    console.log(res.code)
    if (res.code !== 200 && res.code !== 20000) {
      if (res.code == undefined) {
        console.log('进入！！！')
        return res
      }
      Message({
        message: res.msg || res.message || '请求接口报错',
        type: 'error',
        duration: 5 * 1000
      })
      if (res.code === 406) {
        store.dispatch('user/resetToken').then(() => {
          setTimeout(() => {
            location.reload()
          }, 1000)
        })
      }
      if (res.code === 401) {
        window.location.reload()
      }

      return Promise.reject(new Error(res.msg || res.message || '请求Error'))
    } else {
      return res
    }
  },
  error => {
    if (error && error.response && error.response.status == 401) {
      if (error.response.config.url.indexOf('/oauth/token') == -1) {
        window.location.reload()
      }
      return Promise.reject(error)
    } else if (error && error.response && error.response.status == 403) {
      Message({
        message: '您没有权限访问',
        type: 'error',
        duration: 5 * 1000
      })
      return window.location.reload()
    }
    return Promise.reject(error)
  }
)

export default service
